'use strict';

exports.__esModule = true;

var _component = require('../../component.js');

var _component2 = _interopRequireDefault(_component);

var _checkVolumeSupport = require('./check-volume-support');

var _checkVolumeSupport2 = _interopRequireDefault(_checkVolumeSupport);

var _obj = require('../../utils/obj');

require('./volume-bar.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @file volume-control.js
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


// Required children


/**
 * The component for controlling the volume level
 *
 * @extends Component
 */
var VolumeControl = function (_Component) {
  _inherits(VolumeControl, _Component);

  /**
   * Creates an instance of this class.
   *
   * @param {Player} player
   *        The `Player` that this class should be attached to.
   *
   * @param {Object} [options={}]
   *        The key/value store of player options.
   */
  function VolumeControl(player) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, VolumeControl);

    options.vertical = options.vertical || false;

    // Pass the vertical option down to the VolumeBar if
    // the VolumeBar is turned on.
    if (typeof options.volumeBar === 'undefined' || (0, _obj.isPlain)(options.volumeBar)) {
      options.volumeBar = options.volumeBar || {};
      options.volumeBar.vertical = options.vertical;
    }

    // hide this control if volume support is missing
    var _this = _possibleConstructorReturn(this, _Component.call(this, player, options));

    (0, _checkVolumeSupport2['default'])(_this, player);

    // while the slider is active (the mouse has been pressed down and
    // is dragging) we do not want to hide the VolumeBar
    _this.on(_this.volumeBar, ['slideractive'], function () {
      _this.volumeBar.addClass('vjs-slider-active');
      _this.lockShowing_ = true;
    });

    // when the slider becomes inactive again we want to hide
    // the VolumeBar, but only if we tried to hide when
    // lockShowing_ was true. see the VolumeBar#hide function.
    _this.on(_this.volumeBar, ['sliderinactive'], function () {
      _this.volumeBar.removeClass('vjs-slider-active');
      _this.lockShowing_ = false;

      if (_this.shouldHide_) {
        _this.hide();
      }
    });

    // show/hide the VolumeBar on focus/blur
    // happens in VolumeControl but if we want to use the
    // VolumeBar by itself we will need this
    _this.on(_this.volumeBar, ['focus'], function () {
      return _this.show();
    });
    _this.on(_this.volumeBar, ['blur'], function () {
      return _this.hide();
    });
    return _this;
  }

  /**
   * Remove the visual hidden state from the `VolumeControl`.
   */


  VolumeControl.prototype.show = function show() {
    this.shouldHide_ = false;

    if (this.options_.vertical) {
      this.removeClass('vjs-visual-hide-vertical');
    } else {
      this.removeClass('vjs-visual-hide-horizontal');
    }
  };

  /**
   * Hide the `VolumeControl` visually but not from screen-readers unless
   * showing is locked (due to the slider being active). If showing is locked
   * hide will be called when the slider becomes inactive.
   */


  VolumeControl.prototype.hide = function hide() {
    // if we are currently locked to the showing state
    // don't hide, but store that we should hide when
    // lockShowing_ turns to a false value.
    if (this.lockShowing_) {
      this.shouldHide_ = true;
      return;
    }

    // animate hiding the bar via transitions
    if (this.options_.vertical) {
      this.addClass('vjs-visual-hide-vertical');
    } else {
      this.addClass('vjs-visual-hide-horizontal');
    }
  };

  /**
   * Create the `Component`'s DOM element
   *
   * @return {Element}
   *         The element that was created.
   */


  VolumeControl.prototype.createEl = function createEl() {
    var orientationClass = 'vjs-volume-horizonal vjs-visual-hide-horizontal';

    if (this.options_.vertical) {
      orientationClass = 'vjs-volume-vertical vjs-visual-hide-vertical';
    }

    return _Component.prototype.createEl.call(this, 'div', {
      className: 'vjs-volume-control vjs-control ' + orientationClass
    });
  };

  return VolumeControl;
}(_component2['default']);

/**
 * Default options for the `VolumeControl`
 *
 * @type {Object}
 * @private
 */


VolumeControl.prototype.options_ = {
  children: ['volumeBar']
};

_component2['default'].registerComponent('VolumeControl', VolumeControl);
exports['default'] = VolumeControl;
